package xyz.etztech.qol.listeners;

import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.TextComponent;
import org.apache.commons.lang.StringUtils;
import org.bukkit.ChatColor;
import org.bukkit.GameMode;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerCommandPreprocessEvent;
import xyz.etztech.core.CoreUtils;
import xyz.etztech.core.web.CoreWeb;
import xyz.etztech.qol.QoL;
import xyz.etztech.qol.other.LinkCommand;
import org.dynmap.DynmapAPI;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class CommandPreprocessListener implements Listener {

    QoL plugin;
    private Map<UUID, String> confirmTpMap = new HashMap<>();
    private Map<UUID, Boolean> dynmapVisibleStatusMap = new HashMap<>();

    public CommandPreprocessListener(QoL plugin) {
        this.plugin = plugin;
    }

    @EventHandler
    public void onPlayerCommandPreprocess(PlayerCommandPreprocessEvent event) {

        String command = event.getMessage();
        String base = command.split(" ")[0].substring(1).toLowerCase(); // Strip the slash
        String auditBypassPerm = "qol.audit.bypass." + base;

        Player sender = event.getPlayer();
        DynmapAPI dynmap = plugin.getDynmap();

        // Spec dynmap hide
        if (base.equals("spec") && sender.hasPermission("SafeSpectate.spectate") && dynmap != null) {
            boolean visibleStatus;
            if (sender.getGameMode() != GameMode.SPECTATOR){
                dynmapVisibleStatusMap.put(sender.getUniqueId(), dynmap.getPlayerVisbility(sender.getName()));
                visibleStatus = false;
            }
            else if (dynmapVisibleStatusMap.containsKey(sender.getUniqueId())){
                visibleStatus = dynmapVisibleStatusMap.remove(sender.getUniqueId());
            }
            else {
                visibleStatus = true;
            }

            dynmap.setPlayerVisiblity(sender.getDisplayName(), visibleStatus);
        }

        // Spec TP confirmation
        if (sender.hasPermission("qol.tpconfirm")) {
            //check if the command is a tp command
            if (noSlash(base).equals("tp") || noSlash(base).toLowerCase().startsWith("tele") || noSlash(command).toLowerCase().startsWith("lagg tpchunk")) {
                //If the user is in the confirm tp map, remove them and let the command run
                if (command.equals(confirmTpMap.get(sender.getUniqueId()))) {
                    confirmTpMap.remove(sender.getUniqueId());
                }
                //If the user is running the tp command for the first time outside of spec
                else if (sender.getGameMode() != GameMode.SPECTATOR) {
                    //Cancel the command
                    event.setCancelled(true);

                    //Add the user to the tp confirm map
                    confirmTpMap.put(sender.getUniqueId(), command);

                    TextComponent message = new TextComponent(ChatColor.GREEN + "You are TPing out of spec, run command again to confirm.");
                    sender.spigot().sendMessage(message);
                }
            }
        }

        // Command Auditing
        if (sender.hasPermission("qol.audit") && !sender.hasPermission(auditBypassPerm) && plugin.getConfig().getBoolean("audit.enabled")) {
            boolean auditable = false;

            for (String audit : QoL.getAudits()) {
                if (noSlash(command).startsWith(noSlash(audit))) {
                    auditable = true;
                    break;
                }
            }

            if (auditable) {
                Map<String, String> post = new HashMap<>();

                post.put("username", "QoL Auditor");
                post.put("content", "[" + StringUtils.capitalize(sender.getGameMode().name().toLowerCase()) + "] " + sender.getName() + " executed command: " + command);
                String webhook = plugin.getConfig().getString("audit.webhook");
                if (StringUtils.isNotEmpty(webhook)) {
                    CoreWeb.asyncPost(plugin, webhook, post);
                }

            }
        }

        // Links
        for (LinkCommand linkCommand : QoL.getLinks()) {
            if (base.equalsIgnoreCase(linkCommand.getCommand())) {
                event.setCancelled(true);
                TextComponent link = new TextComponent(ChatColor.GREEN + linkCommand.getMessage());
                link.setClickEvent(new ClickEvent(ClickEvent.Action.OPEN_URL, linkCommand.getUrl()));
                sender.spigot().sendMessage(link);
                return;
            }
        }

    }

    public String noSlash(String command) {
        return command.startsWith("/") ? command.substring(1) : command;
    }


}
